import abc
import os
from ..api_v1 import API as API
from ..core import File as File
from ..errors import SpreadsheetError as SpreadsheetError
from _typeshed import Incomplete
from abc import ABC, abstractmethod
from io import BytesIO
from typing import Any, BinaryIO, List, Union

logger: Incomplete

class _NamedCellBaseClass(ABC, metaclass=abc.ABCMeta):
    name: Incomplete
    def __init__(self, name: str) -> None: ...
    @abstractmethod
    def serialize(self) -> dict: ...

class NamedInputCell(_NamedCellBaseClass):
    value: Incomplete
    def __init__(self, name: str, value: Union[bool, int, str, float]) -> None: ...
    def serialize(self, convert_value_to_str: bool = ...) -> dict: ...
    def serialize_for_fill_spreadsheet(self) -> dict: ...

class SpreadsheetCalculationInput(NamedInputCell): ...

class NamedOutputCell(_NamedCellBaseClass):
    def __init__(self, name: str) -> None: ...
    @property
    def result(self) -> Any: ...
    @result.setter
    def result(self, value: Any) -> None: ...
    def equals(self, named_cell_result: dict) -> bool: ...
    def serialize(self) -> dict: ...

class _DirectCellBaseClass(ABC, metaclass=abc.ABCMeta):
    sheet_name: Incomplete
    column: Incomplete
    row: Incomplete
    def __init__(self, sheet_name: str, column: str, row: int) -> None: ...
    @abstractmethod
    def serialize(self) -> dict: ...

class DirectInputCell(_DirectCellBaseClass):
    value: Incomplete
    def __init__(self, sheet_name: str, column: str, row: int, value: Union[bool, int, str, float]) -> None: ...
    def serialize(self, convert_value_to_str: bool = ...) -> dict: ...

class DirectOutputCell(_DirectCellBaseClass):
    def __init__(self, sheet_name: str, column: str, row: int) -> None: ...
    @property
    def result(self) -> Any: ...
    @result.setter
    def result(self, value: Any) -> None: ...
    def equals(self, direct_cell_result: dict) -> bool: ...
    def serialize(self) -> dict: ...

class InputCellRange:
    def __init__(self, sheet_name: str, left_column: str, top_row: int, data: List[List[Union[float, str]]]) -> None: ...
    def serialize(self) -> dict: ...

def stringify_value(value: Union[bool, int, str, float]) -> str: ...

class SpreadsheetResult:
    def __init__(self, *, values: dict = ..., file_content: bytes = ...) -> None: ...
    @property
    def values(self) -> dict: ...
    @property
    def file_content(self) -> bytes: ...
    def get_value(self, name: str) -> Any: ...

class SpreadsheetCalculation:
    def __init__(self, file: Union[BytesIO, File], inputs: List[SpreadsheetCalculationInput]) -> None: ...
    @classmethod
    def from_path(cls, file_path: Union[str, bytes, os.PathLike], inputs: List[SpreadsheetCalculationInput]) -> SpreadsheetCalculation: ...
    def evaluate(self, include_filled_file: bool = ...) -> SpreadsheetResult: ...
    @property
    def result(self) -> SpreadsheetResult: ...

class SpreadsheetTemplate:
    def __init__(self, file: BytesIO, input_cells: List[Union[DirectInputCell, NamedInputCell, InputCellRange]]) -> None: ...
    @classmethod
    def from_path(cls, file_path: Union[str, bytes, os.PathLike], input_cells: List[Union[DirectInputCell, NamedInputCell, InputCellRange]]) -> SpreadsheetTemplate: ...
    def render(self) -> SpreadsheetResult: ...
    @property
    def result(self) -> SpreadsheetResult: ...

def render_spreadsheet(template: BinaryIO, cells: List[Union[DirectInputCell, NamedInputCell, InputCellRange]]) -> File: ...
